package cn.estsh.i3plus.core.apiservice.controller.busi;

import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.platform.bean.SysUser;
import cn.estsh.i3plus.pojo.platform.bean.SysUserInfo;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.exception.ImppExceptionEnum;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @Description : 系统用户操作服务
 * @Reference :
 * @Author : wei.peng
 * @Date : 2018-10-25 21:46
 * @Modify :
 **/
@RestController
@RequestMapping("/sys-user")
@Api(description = "系统用户服务")
public class SysUserController {

    public static final Logger LOGGER = LoggerFactory.getLogger(TaskTimeController.class);

    @Autowired
    private ISysUserService sysUserService;

    @PostMapping(value = "/insert")
    @ApiOperation(value = "新增用户", notes = "新增系统用户")
    public ResultBean insertSysUser(SysUser sysUser,String departmentId,String positionId ,String[] roleIds) {
        try {
            ValidatorBean.beginValid(sysUser)
                            .notNull("userLoginName",sysUser.getUserLoginName())
                            .notNull("userLoginPassword",sysUser.getUserLoginPassword())
                            .notNull("userEmpNo",sysUser.getUserEmpNo())
                            .notNull("userEmail",sysUser.getUserEmpNo());

            ValidatorBean.checkNotNull(departmentId,"部门 id 不能为空");
            ValidatorBean.checkNotNull(positionId,"岗位 id 不能为空");

            if(roleIds == null || roleIds.length <= 0){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("角色信息不能为空")
                        .build();
            }

            //新增初始化
            ConvertBean.modelInitialize(sysUser, AuthUtil.getSessionUser());
            sysUser.setUserStatusId(CommonEnumUtil.USER_STATUS.NORMAL.getValue());
            sysUser.setUserTypeId(CommonEnumUtil.USER_TYPE.USER.getValue());

            SysUser user = sysUserService.insertSysUser(sysUser);

            // 更新用户关系
            sysUserService.refreshRefUserDepartment(user,departmentId,AuthUtil.getSessionUser());
            sysUserService.refreshRefUserPosition(user,positionId,AuthUtil.getSessionUser());
            sysUserService.refreshRefUserRole(user,roleIds,AuthUtil.getSessionUser());

            // 更新冗余信息
            sysUserService.updateSysUser(user);

            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(user);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping(value = "/update")
    @ApiOperation(value = "修改用户", notes = "修改用户信息")
    public ResultBean updateSysUser(SysUser sysUser,String departmentId,String positionId ,String[] roleIds) {
        try {
            ValidatorBean.beginValid(sysUser)
                    .notNull("id",sysUser.getId())
                    .notNull("userLoginName",sysUser.getUserLoginName())
                    .notNull("userLoginPassword",sysUser.getUserLoginPassword())
                    .notNull("userEmpNo",sysUser.getUserEmpNo())
                    .notNull("userEmail",sysUser.getUserEmpNo());

            ValidatorBean.checkNotNull(departmentId,"部门 id 不能为空");
            ValidatorBean.checkNotNull(positionId,"岗位 id 不能为空");

            if(roleIds == null || roleIds.length <= 0){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("角色信息不能为空")
                        .build();
            }

            // 用户信息封装
            SysUserInfo userInfo = sysUserService.getSysUserInfoById(sysUser.getId().toString());
            sysUser.setUserInfo(userInfo);
            ConvertBean.modelUpdate(sysUser, AuthUtil.getSessionUser());
            ConvertBean.modelUpdate(userInfo, AuthUtil.getSessionUser());

            // 更新用户关系
            sysUserService.refreshRefUserDepartment(sysUser,departmentId,AuthUtil.getSessionUser());
            sysUserService.refreshRefUserPosition(sysUser,positionId,AuthUtil.getSessionUser());
            sysUserService.refreshRefUserRole(sysUser,roleIds,AuthUtil.getSessionUser());

            // 更新冗余信息
            sysUserService.updateSysUser(sysUser);

            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultObject(sysUser);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value = "/list")
    @ApiOperation(value = "查询所有用户", notes = "查询所有系统用户")
    public ResultBean findAll() {
        try {
            List<SysUser> list = sysUserService.list();
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(list);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @GetMapping(value="/query-user-by-pager")
    @ApiOperation(value="查询系统用户",notes="组合查询用户，外带分页查询(isValid = 1)")
    public ResultBean querySysUserByPager(SysUser user, Pager pager){
        try{
            // 初始化用户查询讯息
            if(null == user) {
                user = new SysUser();
            }
            user.setIsValid(CommonEnumUtil.IS_VAILD.VAILD.getValue());

            ListPager<SysUser> listPager = sysUserService.querySysUserByPager(user, pager);

            return ResultBean.success("操作成功").setListPager(listPager)
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @DeleteMapping(value = "/delete")
    @ApiOperation(value = "删除定时任务", notes = "删除定时任务")
    public ResultBean delete(String id) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "用户id 不能为空");

            sysUserService.deleteSysUserById(id);
            return ResultBean.success().setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping(value = "/status")
    @ApiOperation(value = "修改用户状态", notes = "根据ID修改用户状态")
    public ResultBean updateStatus(String id, int status) {
        try {
            // 数据校验
            ValidatorBean.checkNotNull(id, "用户id 不能为空");
            ValidatorBean.checkNotZero(status, "用户状态不能为空");

            sysUserService.updateSysUserStatus(id, status, AuthUtil.getSessionUser());
            return new ResultBean(true);
        } catch (ImppBusiException busExcep) {
            LOGGER.error(busExcep.getErrorMsg() + "：{}", busExcep.getErrorDetail(), busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        } catch (Exception e) {
            LOGGER.error(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getDescription() + "：{}", e.getMessage(), e);
            return ResultBean.fail().setCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode());
        }
    }

    @DeleteMapping(value = "/batch-delete")
    @ApiOperation(value = "根据id批量删除", notes = "根据id批量删除")
    public ResultBean deleteBatchSysUserByIds(String[] ids){
        try{
            ids = ConvertBean.modelSafeArrayNumber(ids,true);
            if(ids.length == 0){
                throw ImppExceptionBuilder.newInstance()
                        .setSystemID(CommonEnumUtil.SOFT_TYPE.CORE.getCode())
                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                        .setErrorDetail("请选择需要删除的用户。")
                        .build();
            }

            sysUserService.deleteBatchSysUserById(ids,AuthUtil.getSessionUser());
            return ResultBean.success("删除成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        }catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.SYSTEM_EXCEPTION.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.SYSTEM_EXCEPTION.getCode());
        }
    }

    @PutMapping(value = "/batch-status")
    @ApiOperation(value = "批量修改系统功能状态", notes = "根据ID修改系统功能状态")
    public ResultBean updateBatchStatus(String[] ids,int status) {
        try {
            ids = ConvertBean.modelSafeArrayNumber(ids,true);

            // 数据校验
            ValidatorBean.checkNotNull(ids, "功能id 不能为空");
            ValidatorBean.checkNotZero(status, "功能状态不能为空");

            sysUserService.updateBatchSysUserStatus(ids,status,AuthUtil.getSessionUser());

            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch(ImppBusiException busExcep){
            LOGGER.error(busExcep.getErrorMsg() + "：{}",busExcep.getErrorDetail(),busExcep);
            return ResultBean.fail(busExcep.getErrorShow());
        }catch(Exception e){
            LOGGER.error(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getDescription() + "：{}",e.getMessage(),e);
            return ResultBean.fail().setCode(ImppExceptionEnum.BUSINESS_EXCEPTION_DATA_ERROR.getCode());
        }
    }

}
